/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/


void kmpPlaylist::init() {
    //First we set the playlist tree so it doesn't sort the songs (they need to be in the correct order)
    // and to make sure that it doesn't do some funky resorting
    playlistTree->addColumn("#");
    playlistTree->addColumn("Title");
    playlistTree->addColumn("Time");
    playlistTree->setSorting(-1, TRUE);
    playlistTree->setSelectionMode(QListView::Extended);
    playlistTree->setResizeMode(QListView::NoColumn);
    playlistTree->setColumnWidthMode(1, QListView::Manual);
    playlistTree->setType(0);
    playlistTree->setHScrollBarMode (QScrollView::AlwaysOff);
    playlistTree->setColumnAlignment (2, Qt::AlignRight);
    
    //Then we set the "total time" spot so that it fits the time perfecty so you can have up to
    // 999 hours of music with out cutting off the edges
    totalTimeLabel->setMaximumWidth(QFontMetrics(totalTimeLabel->font()).width(" 000:00:00 "));
    totalTimeLabel->setMinimumWidth(QFontMetrics(totalTimeLabel->font()).width(" 000:00:00 "));
    //And we load the current song playling icon here
    icons * MyIcons;
    MyIcons = new icons;
    playIcon = MyIcons->dotI;
    delete MyIcons;
    //Next we add an "accelerator" this enables the delete key so you can delete songs with it
    QAccel * accel = new QAccel( this );
    accel->connectItem(accel->insertItem (Key_Delete), this, SLOT(deleteKeyPressed()));
    accel->connectItem(accel->insertItem (Key_Space), this, SLOT(spaceKeyPressed()));
    //And we set the current song playling to -1 (So the indicator starts off correctly)
    curSong = -1;
    //And we don't want to have the current song playing until we get the playlist
    pausePaint  = TRUE;
    //And we create a new playlist manager window here
    managerWindow = new kmpPlaylists(this);
    connect(managerWindow, SIGNAL(pauseStatus()), this, SIGNAL(pauseStatus()));
    connect(managerWindow, SIGNAL(unpauseStatus()), this, SIGNAL(unpauseStatus()));
    //And we set the browsewindow to NULL
    browseWindow = NULL;
    playlistSize = 0;
    playlistMax = 0;
    showOnlyFilename = FALSE;
}

//Name:  populatePlaylist
//When&What Calls:  this is called by the main window when it gets a new playlist
//Purpose:  To update the internal tree to show the playlist
void kmpPlaylist::populatePlaylist( linkedList<mpd_SongStore> & theList){
    playlistSize = 0;
    //We have a new playlist, so new time!
    totalTime = 0;
    //pausepaint is set to true so we don't try to say which song is playing before
    // we even have the playlist updated    
    pausePaint = TRUE;
    //We make the tree not visible, so It updates lightning quick
    playlistTree->setHidden(TRUE);
    //And we get rid of the old playlist
    playlistTree->clear();
    //Some temporary variables
    mpd_SongStore  theSong;
    dragListViewItem  * ItemToAdd;
    dragListViewItem * lastItem = 0;
    //Next we loop through every song in the linked list, and put it into the tree
    theList.startFullRead();
    for (unsigned long i = 1 ; i <= theList.size(); i++) {
        //Just in case there is no song at the spot we just skip it
        if(!theList.nextRecord(theSong)) continue;
        //For every song we add a new pointer attached to the tree with the songnumber, title and time
        if(showOnlyFilename) {
            if(lastItem) {
                ItemToAdd = new dragListViewItem(playlistTree, lastItem, theSong, QString::number(i)+". ", fileTitle(theSong.file), minTime(theSong.time));
                lastItem = ItemToAdd;
            } else {
                ItemToAdd = new dragListViewItem(playlistTree,theSong, QString::number(i)+". ", fileTitle(theSong.file), minTime(theSong.time));
                lastItem = ItemToAdd;
            }
        }else {
            if(lastItem) {
                ItemToAdd = new dragListViewItem(playlistTree, lastItem, theSong, QString::number(i)+". ", title(theSong), minTime(theSong.time) );
                lastItem = ItemToAdd;
            } else {
                ItemToAdd = new dragListViewItem(playlistTree, theSong, QString::number(i)+". ", title(theSong), minTime(theSong.time) );
                lastItem = ItemToAdd;
            }
        }
        playlistSize++;
        //And make sure that they can be dragged :)
        ItemToAdd->setDragEnabled(TRUE);
        if(theSong.time > 0) totalTime += theSong.time;
    }
    updatePlaylist();
    //Now that we have the playlist updated we can show it again
    playlistTree->setHidden(FALSE);
    //Right now we have no song highlighted so the "cur song" is -1
    curSong = -1;
    //And let the window paint the current song now that we have the new playlist
    pausePaint = FALSE;
    resetTime();
}

//Name:  playsong
//When&What Calls:  this is called when the user double clicks on a song
//Purpose:  to send a play command to the server of the current song playing
void kmpPlaylist::playsong( QListViewItem * theItem)
{
    pauseStatus();
    //Make sure we don't try to play a song if we aren't connected
    if(conn==NULL) return;
    //We don't really use the integer or the point, but we do stuff to em so we don't get an
    //And we send a playcommand at the current song, which we compute by where its at
    // divided by how many pixels it is high, so we get the position this song is at
    mpd_sendPlayCommand(conn, theItem->itemPos() / theItem->height());
    //Then we do the usual check for errors
    checkForErrors();
    mpd_finishCommand(conn);
    checkForErrors();   
    unpauseStatus();
}
//Name:  checkForErrors
//When&What Calls:  this is called a bazillion times to check the pipe for errors
//Purpose:  To catch any errors and avoid crashes
void kmpPlaylist::checkForErrors()
{
    //If there is an error in our command connection
    if(conn->error) {
        //We get the error..
        QString Error;
        Error = QString(conn->errorStr);
        printf(Error);
        //printf("Error Code: %d, At: %d\n", conn->errorCode, conn->errorAt);
        if(conn->error == MPD_ERROR_ACK) {
            //If its just an acknoledgement error, we just clear it, as its not important
            mpd_clearError(conn);
            return;
        }
        //Otherwise we close the connection, and tell the main window to close the connections
        mpd_clearError(conn);
        mpd_closeConnection(conn);
        conn =NULL;
        //This tells the main window that we have had a connection error
        connError();
    }
}


//Name:  setConn
//When&What Calls:  this is called whenever the main window gets a new connection
//Purpose:  To give the playlist window the new command connection
void kmpPlaylist::setConn( mpd_Connection * conn_)
{
    conn = conn_;
    //And we pass the command connection to the manager window too.
    managerWindow->setConn(conn);
    //And we set the current song playling to -1 (So the indicator starts off correctly)
    curSong = -1;
    //And we don't want to have the current song playing until we get the playlist
    pausePaint  = TRUE;
}

//Name:  songPlaying
//When&What Calls:  this is called by the status checks and passes the current song playing
//Purpose:  To put an indicator next to the current song plaing
void kmpPlaylist::songPlaying( int songNum)
{
    //If the current song playing is already highlighted, skip this
    if (songNum == curSong)  return;
    //If we dont' have the full playlist yet, also skip this
    if(pausePaint) return;
    //Temporary pointer to a item
    QListViewItem  * ItemToEdit = 0;    
    dragListViewItem * realItem;
    //If the song isn't the first one highlighted, then find the previous song highlighted and un highlight it
    if(curSong!= -1)  {
        //Find the last song highlighted and make no indicator on it
        findPointer(curSong, playlistTree, ItemToEdit);
        realItem = (dragListViewItem*)ItemToEdit;
        if(realItem) realItem->setPlaying(0);
        realItem->repaint();
    }
    //Now find the song that should have an indicator on it, and put it on it
    findPointer(songNum, playlistTree, ItemToEdit);
    realItem = (dragListViewItem*)ItemToEdit;
    if(realItem) realItem->setPlaying(1);
    realItem->repaint();
    curSongPlaying = realItem;
    //And now we store that this song has the indicator
    curSong = songNum;
}

//Name:  findPointer
//When&What Calls:  this is called by functions to find the nth spot item in the tree
//Purpose:  to find an accurate nth spot item
void kmpPlaylist::findPointer( const int pos, QListView * theTree, QListViewItem * & theItem )
{
    //The first item is the first child of the tree
    theItem = theTree->firstChild();
    for(int i = 0; i < pos; i++) {
        //If there is no item we just return
        if(!theItem) return;
        //And we move down the list one item
        theItem = theItem->itemBelow();
    }
}

//Name:  deleteKeyPressed
//When&What Calls:  this is called when a user presses the delete key anywhere on the window
//Purpose:  To delete items from the playlist
void kmpPlaylist::deleteKeyPressed()
{
    //Some temporary variables to store pointers
    mpd_SongStore theSong;
    QListViewItem * ItemAt;
    QListViewItem * ItemToDelete;
    //We start at the end... VERY important!
    ItemAt = playlistTree->lastItem();
    linkedList<kmpCommand> myCommandList;
    //And we loop through EVERY item in the list
    while(ItemAt != 0) {
        //If the item is selected, then we delete it from the playlist tree
        if(ItemAt->isSelected()) {
            ItemToDelete = ItemAt;
            //We increment the playlist, because we are sending a command that will increment the
            // the server playlist. We do this so we don't get a new playlist everytime we delete
            // something
            if(ItemToDelete->itemPos() / ItemToDelete->height() == curSong) curSong = -1;
            
            //Sending the delete command
            
            kmpCommand myCommand;
            myCommand.theCommand = KMP_COMMAND_DEL;
            myCommand.songNum = ItemToDelete->itemPos() / ItemToDelete->height();
            myCommand.theItem = (dragListViewItem * ) ItemToDelete;
            myCommandList.add(myCommand);
            
            //And then move up the list
            ItemAt = ItemAt->itemAbove();
            //And get rid of the item in the widget tree
            playlistTree->takeItem(ItemToDelete);
            //And move on!
            continue;
        }
        //If it isn't selected then we just move up!
        ItemAt = ItemAt->itemAbove();
    } 
    executeList(myCommandList);
}

//Name:  fixTree
//When&What Calls:  this is called when the tree needs to be fixed
//Purpose:  to fix the numbers of the items so they are in order
void kmpPlaylist::fixTree()
{
    QListViewItem * ItemAt;
    //The first track to start on it 1
    int CurrentNumber = 1;
    //We get the first item (the first child of the tree)
    ItemAt = playlistTree->firstChild();
    //And loop through each one setting a new value for the track number
    while(ItemAt !=0) {
        ItemAt->setText(0, QString::number(CurrentNumber++)+". ");
        ItemAt = ItemAt->itemBelow();
    }
    updatePlaylist();
}

//Name:  closeEvent
//When&What Calls: this is called when the playlist gets the close event 
//Purpose:  To tell the main window that we have closed
void kmpPlaylist::closeEvent( QCloseEvent * a) {
    //This is connected to the main window and notifies it that we are closing
    playlistClosed();
    //Accept the close event
    a->accept();
}

//Name:  sendShuffle
//When&What Calls:  this is called when the user pressed the shuffle button
//Purpose:  To send the shuffle command
void kmpPlaylist::sendShuffle()
{
    //During any commands sent through the pipe we have to pause the status update
    // So we don't do 2 things at once :)
    pauseStatus();
    //Send the shuffle command to the server
    mpd_sendShuffleCommand(conn);
    //Check for errors
    checkForErrors();
    mpd_finishCommand(conn);
    checkForErrors();
    unpauseStatus();
}

//Name:  sendClear
//When&What Calls:  this is called when the user pressed the clear button
//Purpose:  To send the playlist clear command
void kmpPlaylist::sendClear()
{
    //During any commands sent through the pipe we have to pause the status update
    // So we don't do 2 things at once :)
    pauseStatus();
    //Send the clear playlist command
    mpd_sendClearCommand(conn);
    //Check for errors
    checkForErrors();
    mpd_finishCommand(conn);
    checkForErrors();
    unpauseStatus();
}

//Name:  openManage
//When&What Calls: this is called when the user asks to open the playlist manager window 
//Purpose:  To show the playlist manager window
void kmpPlaylist::openManage()
{
    //Tell the manager window to show it self
    managerWindow->show();
}



//Name:  resetTime
//When&What Calls:  This is called when ever the total time is changed
//Purpose:  To reset the total time and the width of the label widget
void kmpPlaylist::resetTime()
{
    //Set the text to the new time
    totalTimeLabel->setText(hrTime(totalTime));
    //And make the label fit it perfectly
    totalTimeLabel->setMaximumWidth(QFontMetrics(totalTimeLabel->font()).width(hrTime(totalTime)));
    totalTimeLabel->setMinimumWidth(QFontMetrics(totalTimeLabel->font()).width(hrTime(totalTime)));
}

//Name: enableMe
//When&What Calls: this is called by the main window
//Purpose: To change the status of the interface
void kmpPlaylist::enableMe( bool status )
{
    //Change our status of our widgets
    playlistTree->setEnabled(status);
    btn_shuffle->setEnabled(status);
    btn_clear->setEnabled(status);
    totalTimeLabel->setEnabled(status);
    //And Pass it on to our children
    managerWindow->enableMe(status);
}



void kmpPlaylist::setBrowse( kmpBrowse * theWindow )
{
    browseWindow = theWindow;
}


void kmpPlaylist::addsong(QString MType, dragListView * source, dragListViewItem* loc) {
    tempList.reset();
    int curLoc = -1;
    if(MType == "kmp_file") 
    {
        mpd_SongStore theSong;
        dragListViewItem * ItemAt;
        dragListViewItem * ItemToAdd;
        ItemAt = (dragListViewItem *) (source->firstChild());
        linkedList<kmpCommand> commandList;
        //And we loop through EVERY item in the list
        int incrementalNum = 0;
        while(ItemAt != 0) 
        {
            //If the item is selected, then we add it
            if(ItemAt->isSelected()) 
            {
                theSong = ItemAt->getItem();
                kmpCommand addCommand;
                kmpCommand movCommand;
                if(loc)
                {
                    movCommand.theCommand = KMP_COMMAND_MOV;
                    dragListViewItem * locItem;
                    //The Location it should be at;
                    curLoc = loc->itemPos() / loc->height();
                    //The Item to add After
                    locItem = (dragListViewItem*) loc->itemAbove();
                    //If there is no Item to add after, then its at the top of the playlist
                    if(locItem) 
                    {
                        //Add after the item -> locItem
                        if(showOnlyFilename) {
                            ItemToAdd = new dragListViewItem(playlistTree, locItem, theSong, QString::number(playlistSize + incrementalNum++) + ". " , fileTitle(theSong.file), minTime(theSong.time) );
                        }else {
                            ItemToAdd = new dragListViewItem(playlistTree, locItem, theSong, QString::number(playlistSize + incrementalNum++) + ". " , title(theSong), minTime(theSong.time));
                        }           
                    } else {
                        //Add at the Beginning
                        if(showOnlyFilename) {
                            ItemToAdd = new dragListViewItem(playlistTree, theSong, QString::number(playlistSize + incrementalNum++) + ". " , fileTitle(theSong.file), minTime(theSong.time) );
                        }else {
                            ItemToAdd = new dragListViewItem(playlistTree, theSong, QString::number(playlistSize + incrementalNum++) + ". " , title(theSong), minTime(theSong.time));
                        }
                    }
                    locItem = ItemToAdd;
                } else { 
                    movCommand.theCommand = KMP_COMMAND_NO;
                    //Add it at the end
                    if(showOnlyFilename) {
                        ItemToAdd = new dragListViewItem(playlistTree, playlistTree->lastItem(), theSong, QString::number(playlistSize + incrementalNum++) + ". " , fileTitle(theSong.file), minTime(theSong.time) );
                    }else {
                        ItemToAdd = new dragListViewItem(playlistTree, playlistTree->lastItem(), theSong, QString::number(playlistSize + incrementalNum++) + ". " , title(theSong), minTime(theSong.time));
                    }
                }
                ItemToAdd->setDragEnabled(TRUE);
                //Post commands to do:
                addCommand.theCommand = KMP_COMMAND_ADD;
                addCommand.theSong = theSong;
                addCommand.theItem = ItemToAdd;
                commandList.add(addCommand);
                if (movCommand.theCommand != KMP_COMMAND_NO) 
                {
                    movCommand.theSong = theSong;
                    movCommand.theItem = ItemToAdd;
                    movCommand.songNum = playlistSize + incrementalNum -1;
                    movCommand.songTo = curLoc++;
                    commandList.add(movCommand);
                }
                //Continue to next item to add
                ItemAt = (dragListViewItem *) ItemAt->itemBelow();
                continue;
            }
            //If it isn't selected then we just move down
            ItemAt = (dragListViewItem * ) ItemAt->itemBelow();
        }
        executeList(commandList);
        
        
        
    } else if(MType == "kmp_dir") {
        QListViewItemIterator it(source);
        dragListViewItem * ItemToAdd;
        while (it.current()) {
            if (it.current()->isSelected() ) {
                int incrementalNum = 0;
                mpd_InfoEntity * entity;
                pauseStatus();
                mpd_sendListallInfoCommand(conn, (buildDirectoryPath(it.current())).utf8());
                while((entity = mpd_getNextInfoEntity(conn))) {
                    //If for some really wierd reason the information comming in the
                    // pipe isn't a song, we skip it, and hope we get what we
                    // want. This should NEVER happen, but b.s.t.s.
                    if(entity->type!=MPD_INFO_ENTITY_TYPE_SONG) {
                        mpd_freeInfoEntity(entity);
                        continue;
                    }
                    //Then we create a new storable song (non pointers used)
                    mpd_SongStore aSong;
                    //And set all of the information to it.     
                    aSong.file = QString::fromUtf8(entity->info.song->file);
                    aSong.artist = QString::fromUtf8(entity->info.song->artist);
                    aSong.album = QString::fromUtf8(entity->info.song->album);
                    aSong.title = QString::fromUtf8(entity->info.song->title);
                    aSong.track = QString::fromUtf8(entity->info.song->track);
                    aSong.name = QString::fromUtf8(entity->info.song->name);
                    aSong.time = entity->info.song->time;
                    //And since we updated the playlist, we need to tell the scroll to
                    // update its information.
                    //Now we release the memory that we used to get this new song's info
                    tempList.add(aSong);
                    mpd_freeInfoEntity(entity);
                    //And we again check for errors. Just in case :-P
                    checkForErrors();
                }
                unpauseStatus();
                linkedList<kmpCommand> commandList;
                for(unsigned long  i =0; i < tempList.size(); i++) 
                {
                    mpd_SongStore theSong;
                    if(tempList.at(i, theSong))  {
                        kmpCommand addCommand;
                        kmpCommand movCommand;
                        if(loc)
                        {
                            movCommand.theCommand = KMP_COMMAND_MOV;
                            dragListViewItem * locItem;
                            //The Location it should be at;
                            curLoc = loc->itemPos() / loc->height();
                            //The Item to add After
                            locItem = (dragListViewItem*) loc->itemAbove();
                            if(locItem) 
                            {
                                //Add after the item -> locItem
                                if(showOnlyFilename) {
                                    ItemToAdd = new dragListViewItem(playlistTree, locItem, theSong, QString::number(playlistSize + incrementalNum++) + ". " , fileTitle(theSong.file), minTime(theSong.time) );
                                }else {
                                    ItemToAdd = new dragListViewItem(playlistTree, locItem, theSong, QString::number(playlistSize + incrementalNum++) + ". " , title(theSong), minTime(theSong.time));
                                }           
                            } else {
                                //Add at the Beginning
                                if(showOnlyFilename) {
                                    ItemToAdd = new dragListViewItem(playlistTree, theSong, QString::number(playlistSize + incrementalNum++) + ". " , fileTitle(theSong.file), minTime(theSong.time) );
                                }else {
                                    ItemToAdd = new dragListViewItem(playlistTree, theSong, QString::number(playlistSize + incrementalNum++) + ". " , title(theSong), minTime(theSong.time));
                                }
                            }
                            locItem = ItemToAdd;
                        } else { 
                            movCommand.theCommand =KMP_COMMAND_NO;
                            //Add it at the end
                            if(showOnlyFilename) {
                                ItemToAdd = new dragListViewItem(playlistTree, playlistTree->lastItem(), theSong, QString::number(playlistSize + incrementalNum++) + ". " , fileTitle(theSong.file), minTime(theSong.time) );
                            }else {
                                ItemToAdd = new dragListViewItem(playlistTree, playlistTree->lastItem(), theSong, QString::number(playlistSize + incrementalNum++) + ". " , title(theSong), minTime(theSong.time));
                            }
                        }
                        ItemToAdd->setDragEnabled(TRUE);
                        //Post commands to do:
                        addCommand.theCommand = KMP_COMMAND_ADD;
                        addCommand.theSong = theSong;
                        addCommand.theItem = ItemToAdd;
                        commandList.add(addCommand);
                        if (movCommand.theCommand != KMP_COMMAND_NO) 
                        {
                            movCommand.theSong = theSong;
                            movCommand.theItem = ItemToAdd;
                            movCommand.songNum = playlistSize + incrementalNum -1;
                            movCommand.songTo = curLoc++;
                            commandList.add(movCommand);
                        }
                        
                    }
                }
                executeList(commandList);
            }
            ++it;
        }
    } else if(MType == "kmp_pl") {
        int baseLoc=1;
        int afterOffset = 0; //How far we need to offset when we are after the insert area
        dragListViewItem * ItemAt;  //Where we are looking (selected songs)
        dragListViewItem * AddAfter; //Where to add after
        dragListViewItem * nextItem; //Cache for adding after
        mpd_SongStore theSong; //To help with error Correction
        ItemAt = (dragListViewItem *) (source->firstChild());
        
        linkedList<kmpCommand> commandList;
        if(loc) {
            baseLoc = loc->itemPos() / loc->height();
            AddAfter =  (dragListViewItem*) loc->itemAbove(); //To add under it.
            if(AddAfter && AddAfter->isSelected()) return;  //If we are trying to add below a song selected we don't
        } else {
            baseLoc = playlistSize;
            AddAfter = (dragListViewItem*) source->lastItem();
        }
        //And we loop through EVERY item 
                while(ItemAt != 0) 
        {        //If the item is selected, then we move it
            if(ItemAt->isSelected()) 
            {
                theSong = ItemAt->getItem();
                kmpCommand movCommand;
                movCommand.theCommand = KMP_COMMAND_MOV;                
                int currentLocation = ItemAt->itemPos() / ItemAt->height();
                if(currentLocation < baseLoc) {
                    movCommand.theSong = theSong;
                    movCommand.theItem = ItemAt;
                    movCommand.songNum = currentLocation;
                    movCommand.songTo = baseLoc - 1;
                    commandList.add(movCommand);         
                    nextItem = (dragListViewItem * ) ItemAt->itemBelow();
                    ItemAt->moveItem(AddAfter);
                    ItemAt->setSelected(false);
                    AddAfter = ItemAt;
                    ItemAt = nextItem;
                } else {
                    movCommand.theSong = theSong;
                    movCommand.theItem = ItemAt;
                    movCommand.songNum = currentLocation;
                    movCommand.songTo = baseLoc + afterOffset++;
                    commandList.add(movCommand);
                    if(AddAfter) {
                        ItemAt->moveItem(AddAfter);
                    } else {
                        source->takeItem(ItemAt);
                        source->insertItem(ItemAt);
                    }
                    AddAfter = ItemAt;
                    ItemAt = (dragListViewItem * ) ItemAt->itemBelow();
                }    
            } else {
                ItemAt = (dragListViewItem * ) ItemAt->itemBelow();
            }
            //If it isn't selected then we just move down
            
        }
        executeList(commandList);
    }
}




void kmpPlaylist::showFilename( bool showFilename)
{
    showOnlyFilename = showFilename;
}



void kmpPlaylist::resizeE( QResizeEvent * e )
{
    fixView(e->size().width());
}


void kmpPlaylist::updatePlaylist()
{
    fixView(playlistTree->width());
}


void kmpPlaylist::spaceKeyPressed()
{
    if(curSongPlaying) playlistTree->ensureItemVisible(curSongPlaying);
}


void kmpPlaylist::fixView(int width)
{
    int size = playlistTree->lastItem()->itemPos();
    if(playlistTree->lastItem()) size+=playlistTree->lastItem()->height();
    int viewWidth = playlistTree->viewportSize(width, size).width();
    playlistTree->setColumnWidth(1, viewWidth - playlistTree->columnWidth(0) - playlistTree->columnWidth(2));
}

void kmpPlaylist::executeList(linkedList<kmpCommand>  & theList)
{
    pauseStatus();
    while(theList.size() > 0)
    {
        mpd_sendCommandListBegin(conn);
        unsigned long spot = 0;
        while(spot < theList.size())  {
            kmpCommand curCommand;
            if(!(theList.at(spot, curCommand))) {
                printf("INTERNAL ERROR->Can't Find %lu\n", spot);
                return;
            }
            
            if(curCommand.theCommand == KMP_COMMAND_ADD)
            {
                
                //Check to see if we are trying to add to much
                if(playlistMax && playlistSize +1  > playlistMax) 
                {
                    kmpCommand nextCommand;
                    if(theList.size() > spot + 1) {
                        if(!(theList.at(spot + 1, nextCommand))) printf("ERROR!!!! Not bigger then spot!\n");                        
                        if(nextCommand.theCommand == KMP_COMMAND_MOV)
                        {
                            //Remove Item from Playlist);
                            playlistTree->takeItem(curCommand.theItem);
                            //Delete the add and the mov
                            if(!(theList.at(spot, curCommand))) printf("ERROR Can't get song0!!!!\n");
                           // printf("Removing Song: ");
                           // printf(curCommand.theSong.title);
                          //  printf("\n");           
                            theList.remove(spot);
                            if(!(theList.at(spot, curCommand))) printf("ERROR Can't Get song 1!!!!\n");
                         //   printf("Removing Song Move: ");
                        //    printf(curCommand.theSong.title);
                        //    printf("  From: %d,  To: %d\n", curCommand.songNum, curCommand.songTo);   
                            theList.remove(spot);                      
                            continue;
                        }
                    }
                    //Remove Item from Playlist
                    //   printf("932Removing :  %08x\n", curCommand.theItem);
                    playlistTree->takeItem(curCommand.theItem);
                    //Delete the lone Add
                    theList.remove(spot);
                    continue;
                }
                totalTime+=curCommand.theSong.time;
                incrementPlaylist(); 
                playlistSize++;
                
               // printf("Adding Song: ");
               // printf(curCommand.theSong.file.utf8());
               // printf("\n");
                //printf("\n");
               // printf("PlaylistSize : %d\n", playlistSize);
                mpd_sendAddCommand(conn, curCommand.theSong.file.utf8());
                spot++;
            }
            else if(curCommand.theCommand == KMP_COMMAND_DEL)
            {
                mpd_SongStore theSong;
                //Subtract time from Total Time;
                dragListViewItem * myItem = (dragListViewItem *) curCommand.theItem;
                theSong = myItem->getItem();
                //subtracting the time from total time
                totalTime-=theSong.time;
                incrementPlaylist();
                mpd_sendDeleteCommand(conn, curCommand.songNum);
                playlistSize--;
                spot++;
            }
            else if(curCommand.theCommand == KMP_COMMAND_MOV)
            {
                incrementPlaylist();
                //printf("Moving Song: ");
                //printf(curCommand.theSong.title);
                //printf("  From: %d,  To: %d\n", curCommand.songNum, curCommand.songTo);   
                mpd_sendMoveCommand(conn, curCommand.songNum, curCommand.songTo);
                spot++;
            }
        }
        mpd_sendCommandListEnd(conn);
        //Finish it all
        mpd_finishCommand(conn);
        //Check For Errors
        if(conn->error)
        {
            printf("Error Code: %d, At: %d\n", conn->errorCode, conn->errorAt);
            kmpCommand myCommand;
            if(!(theList.at(conn->errorAt, myCommand))) printf("INTERNAL ERROR->executeList\n");
            printf("Song Name: ");
            printf(myCommand.theSong.file);
            printf("\n");
            if(myCommand.theCommand == KMP_COMMAND_MOV) {
                printf("  From: %lu,  To: %lu\n", myCommand.songNum, myCommand.songTo);
            }
            if(conn->error != MPD_ERROR_ACK)
            {
                //Otherwise we close the connection, and tell the main window to close the connections
                mpd_clearError(conn);
                mpd_closeConnection(conn);
                conn =NULL;
                //This tells the main window that we have had a connection error
                connError();
                return;  
            }
            
            //Remove Completed Commands from our Queue
            for(int delCommand = (conn->errorAt -1); delCommand >= 0; delCommand--)
            {
                theList.remove(delCommand);
            }
            //Check Errors            
            if(conn->errorCode == MPD_ACK_ERROR_PLAYLIST_MAX)
            {
                //Remove the Add command that Caused the Error
                kmpCommand curCommand;
                if(!(theList.at(0, curCommand))) printf("INTERNAL ERROR->executeList\n");
                if(curCommand.theCommand != KMP_COMMAND_ADD) printf("NON ADD returned MAX PLAYLIST\n");
                //Store the Max Size  
                //remove rest of commands, and removes each item from playlist
                for(unsigned long delit = 0; delit < theList.size(); delit++)
                {
                    //Retrieve
                    if(!(theList.at(0, curCommand))) printf("INTERNAL ERROR->executeList\n");
                    decrementPlaylist();
                    if(curCommand.theCommand == KMP_COMMAND_ADD) {
                        //    printf("879Removing :  %08x\n", curCommand.theItem);
                        playlistTree->takeItem(curCommand.theItem);
                        theList.remove(0);
                        playlistSize--;
                        totalTime-=curCommand.theSong.time;
                    }
                } 
                theList.reset();
                playlistMax = playlistSize;
                //     printf("Max playlist size: %d\n", playlistMax);
            } else if (conn->errorCode == MPD_ACK_ERROR_NO_EXIST)
            {                
                kmpCommand curCommand;
                if(!(theList.at(0, curCommand))) printf("INTERNAL ERROR->executeList\n");
                //Check for a Move command directly after, if so Remove it (its for the previous add Command)
                if(curCommand.theCommand ==  KMP_COMMAND_ADD)                
                {
                    printf("Deleting Add...\n");
                    kmpCommand nextCommand;
                    if(theList.size() > 1) 
                    {
                        printf(" > 1\n");
                        if(!(theList.at(1, nextCommand))) printf("ERROR!!!! Not bigger then spot!\n");
                        printf("NextCommand: %d\n", nextCommand.theCommand);
                        printf("Equal to:%d\n", KMP_COMMAND_MOV);
                        if(nextCommand.theCommand == KMP_COMMAND_MOV)
                        {
                            printf("Deleting Add & Move...\n");
                            //Remove Item from Playlist);
                            playlistTree->takeItem(curCommand.theItem);
                            //Delete the add and the mov
                            if(!(theList.at(0, curCommand))) printf("ERROR Can't get song0!!!!\n");
                            printf("Removing Song: ");
                            printf(curCommand.theSong.file);
                            printf("\n");           
                            theList.remove(0);
                            if(!(theList.at(0, curCommand))) printf("ERROR Can't Get song 1!!!!\n");
                            printf("Removing Song Move: ");
                            printf(curCommand.theSong.file);
                            printf("  From: %lu,  To: %lu\n", curCommand.songNum, curCommand.songTo);   
                            theList.remove(0);
                            decrementPlaylist();
                            playlistSize--;
                            totalTime-=curCommand.theSong.time;
                        } else
                        { 
                            printf("Deleteing the plain ADD\n");
                            //Remove Item from Playlist
                            //     printf("913Removing :  %08x\n", curCommand.theItem);
                            playlistTree->takeItem(curCommand.theItem);
                            //Delete the lone Add
                            theList.remove(0);
                            decrementPlaylist();
                            playlistSize--;
                            totalTime-=curCommand.theSong.time;
                        }              
                    } else
                    {
                        printf("Deleteing the LONE ADD\n");
                        //Remove Item from Playlist
                        //     printf("913Removing :  %08x\n", curCommand.theItem);
                        playlistTree->takeItem(curCommand.theItem);
                        //Delete the lone Add
                        theList.remove(0);
                        decrementPlaylist();
                        playlistSize--;
                        totalTime-=curCommand.theSong.time;
                    }
                } else
                {
                    //Remove Item from Playlist
                    //  printf("922Removing :  %08x\n", curCommand.theItem);
                    playlistTree->takeItem(curCommand.theItem);
                    //Delete the single command (either a single add, or a delete command)
                    if(!(theList.remove(0))) printf("INTERNAL - CAN'T DELETE 0\n");
                    decrementPlaylist();
                }
            } else if (conn->errorCode == MPD_ACK_ERROR_PERMISSION)
            {
                kmpCommand curCommand;
                for(unsigned long delit = 0; delit < theList.size(); delit++)
                {
                    if(!(theList.at(0, curCommand))) printf("INTERNAL ERROR->executeList\n");
                    decrementPlaylist();
                    if(curCommand.theCommand == KMP_COMMAND_ADD) 
                    {
                        //   printf("937Removing :  %08x\n", curCommand.theItem);
                        playlistTree->takeItem(curCommand.theItem);
                    }
                } 
                theList.reset();
                printf("NO PERMISSION\n");
            } else {
                printf("UNKNOWN ERROR: %s\n", conn->errorStr);
            }
            //And re-decrement the playlist
            for(unsigned long reDec = 0; reDec < theList.size(); reDec++)
            {
                kmpCommand curCommand;
                if(!(theList.at(0, curCommand))) printf("INTERNAL ERROR->executeList\n");
                //If command is a delete Command, add back time
                if(curCommand.theCommand == KMP_COMMAND_DEL) 
                {
                    mpd_SongStore theSong;
                    //Add time back to  Total Time;
                    dragListViewItem * myItem = (dragListViewItem *) curCommand.theItem;
                    theSong = myItem->getItem();
                    //Adding time back to total time
                    totalTime+=theSong.time;
                } else if (curCommand.theCommand == KMP_COMMAND_ADD)
                {
                    //Subtract back the time from added songs
                    totalTime-=curCommand.theSong.time;
                    playlistSize--;
                }
                decrementPlaylist();
            }
            mpd_clearError(conn);
        } else {
            //No Errors, All commands Completed Successfully
            theList.reset();
        }
    }
   // printf("PlaylistSize: %d\n", playlistSize);
    unpauseStatus();
    resetTime();
    fixTree();
}


void kmpPlaylist::playlistDownload()
{
    playlistTree->clear();
    
    QListViewItem * ItemToAdd = new QListViewItem(playlistTree ," ", "New Playlist Currenty Dowloading... Please Wait", " ");
    ItemToAdd->setEnabled(FALSE);
    
    
}
